# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# creolewrap.py
#  
# classe abstraite permettant de gérer les configurations
# creole1 et creole2 dans zephir
#       
###########################################################################

import os
# import creole1 et creole2
import dico, base64
from copy import copy
from creole.cfgparser import EoleDict
from creole.error import TypeEoleError
from zephir.config import charset
from creole import eosfunc
from glob import glob

class ZephirDict:
    """sert de wrapper par dessus les dictionnaires creole1 et creole2
    """

    def __init__(self, dicos=[], confdir='', mode='config', version='creole2'):
        """initialise un objet dictionnaire avec la bonne api
        """
        self.version = version
        config_file = ""
        if confdir != '':
            if mode == 'modif_config' and os.path.isfile(confdir+os.sep+'zephir.eol'):
                config_file = confdir+os.sep+'zephir.eol'
            elif mode in ['config','modif_config','modif_dico']:
                if os.path.isfile(confdir+os.sep+'dico.eol'):
                    config_file = confdir+os.sep+'dico.eol'
                elif os.path.isfile(confdir+os.sep+'module.eol'):
                    config_file = confdir+os.sep+'module.eol'
        if self.version == 'creole1':
            # initialisiation du dictionnaire
            self.dico = dico.DicoEole(dico_zephir=dicos)
            # dictionnaire des valeurs par défaut ou déjà remplies
            dico_actuel = None
            if config_file != "":
                fic = open(config_file)
                data = fic.readlines()
                lines = [ unicode(line,'ISO-8859-1').encode(charset) for line in data ]
                fic.close()
                # on parse ce dico
                dico_actuel=dico.DicoEole(dico_zephir=[lines]).ParseDico()
            
            if dico_actuel is not None:
                # des valeurs ont déjà été saisies
                dico_final = {}
                # on réinsère les données existantes dans le dictionnaire
                for variable, data in self.dico.dictionnaire.items():
                    # si la variable existait déjà, on reprend sa valeur
                    if dico_actuel.has_key(data[0]):
                        data[1] = dico_actuel[data[0]][0]
                    dico_final[variable] = data
                # on remet le dictionnaire modifié en place dans self.dico
                self.dico.dictionnaire = dico_final
            self.liste_vars = self.dico.liste_vars

        elif self.version == 'creole2':
            # initialisiation du dictionnaire
            self.dico = EoleDict(logfile='/dev/null')
            self.dicos = []
            # chargement de tous les dictionnaires disponibles
            for creoledir in dicos:
                if os.path.isdir(creoledir):
                    self.dicos.extend(glob(creoledir + '/*.xml'))
                    self.dico.read_dir(creoledir)
            # lecture des valeurs actuelles selon le mode demandé
            if config_file != "":
                self.dico.load_values(config_file)
            elif os.path.isfile(confdir+os.sep+'module.eol'):
                # on prend les valeurs par défaut du module
                self.dico.load_values(confdir+os.sep+'module.eol')
            # création d'un liste pour ordonner les variables
            self.index = 0
            self.vars = []
            for family in self.dico.family_order:
                for var in self.dico.families[family]['vars']:
                    self.vars.append(var)
            # création de self.liste_vars
            ind = 0
            self.liste_vars = {}
            for var in self.vars:
                self.liste_vars[var] = ind
                ind += 1

    def init_from_zephir(self,dicos):
        """charge les dictionnaire depuis une liste de chaines xml
        @param dicos: contenu des dictionnaires à implémenter + valeurs actuelles en fin de liste
        """
        if self.version == 'creole1':
            dicos = base64.decodestring(dicos)
            dicos = [dicos.splitlines(True)]
            self.dico = dico.DicoEole(dico_zephir=dicos)
            self.liste_vars = self.dico.liste_vars
        elif self.version == 'creole2':
            # initialisiation du dictionnaire
            self.dico = EoleDict(logfile='/dev/null')
            self.dicos = [base64.decodestring(dico_zeph) for dico_zeph in dicos[:-1]]
            # chargement de tous les dictionnaires disponibles
            for xml_dict in self.dicos:
                self.dico.read_string(xml_dict)
            # chargement des valeurs actuelles
            self.dico.store._sections = eval(dicos[-1])
            self.dico._load_values()
            # création d'un liste pour ordonner les variables
            self.index = 0
            self.vars = []
            for family in self.dico.family_order:
                for var in self.dico.families[family]['vars']:
                    self.vars.append(var)
            ind = 0
            self.liste_vars = {}
            for var in self.vars:
                self.liste_vars[var] = ind
                ind += 1

    def get_value(self,var):
        return self.dico.get_value(var)

    def get_enumeration(self, var):
        vals = []
        allow_other = False
        if self.version == 'creole1':
            conditions = self.dico.dictionnaire[self.dico.liste_vars[var]][3]
            # recherche des enumérations
            for cond in conditions:
                if cond.startswith('enumeration'):
                    try:
                        vals = eval(cond.split('??')[1])
                    except:
                        # si rien n'est spécifié, on propose oui,non par défaut
                        vals = ['oui','non']
                    break
        elif self.version == 'creole2':
            eolvar = self.dico.get_variable(var)
            checks = eolvar.checks
            for check in checks:
                if check[0] == u'valid_enum':
                    # récupération des valeurs possibles
                    for param in check[1]:
                        if param['name'] == 'checkval':
                            if param['value'] == "False":
                                allow_other = True
                        else:
                            vals.extend(eval(param['value']))
        return vals, allow_other

    def get_first(self):
        """retourne les paramètres de la variable nom,valeur,libelle,variable cachée 
        """
        if self.version == 'creole1':
            self.dico.get_first()
        elif self.version == 'creole2':
            self.index = 0
        return self._get_var_details()

    def get_next(self):
        if self.version == 'creole1':
            self.dico.get_next()
        elif self.version == 'creole2':
            self.index = self.index + 1
        try:
            return self._get_var_details()
        except:
            raise dico.PositionError, 'fin de la liste des variables'

    def get_var(self, varname=None):
        """retourne les paramètres d'une variable creole1/2
        """
        return self._get_var_details(varname)

    def _get_var_details(self, varname=None):
        """adapte la représentation des variables creole1/2 pour zephir
        """
        # récupération des infos
        if self.version == 'creole2':
            if varname != None:
                var = self.dico.get_variable(varname)
                self.index = self.vars.index(varname)
            else:
                var = self.dico.get_variable(self.vars[self.index])
            obligatoire = False
            if 'obligatoire' in [ check[0] for check in var.checks ]:
                obligatoire = True
            mime = var.mime
            val = var.get_value()
            if type(val) != list:
                val = [val]
            if val ==[]:
                val = ['']
            data = var.name, val, var.description, var.hidden, obligatoire, mime
        elif self.version == 'creole1':
            if varname != None:
                name, val, description, hidden = self.dico.get_var(varname)
            else:
                name, val, description, hidden = self.dico.get_current()
            checks = self.dico.dictionnaire[self.dico.liste_vars[name]][3]
            if 'obligatoire' in checks:
                obligatoire = True
                liste_funcs = copy(checks)
                liste_funcs.remove('obligatoire')
                # liste_funcs = checks.remove('obligatoire')
            else:
                obligatoire = False
                liste_funcs = checks
            data = name, [val], description, hidden, obligatoire, liste_funcs

        return data

    def get_prec_value(self, varname):
        """ renvoit la valeur précédente """
        if self.version == 'creole2':
            try:
                value = self.dico.get_prec_value(varname)[0]
            except:
                value = ''
                # XXX FIXME : gestion des variables multivaluées !!
                # (retourner aussi une liste pour creole 1)
        elif self.version == 'creole1':
            try:
                value = self.dico.get_var(varname)[1]
            except:
                value = ''
        return value

    def get_default_value(self, varname):
        """ renvoit la valeur par défaut """
        if self.version == 'creole2':
            try:
                value = self.dico.get_value(varname, default=True)[0]
            except:
                value = ''
                # XXX FIXME : gestion des variables multivaluées !!
                # (retourner aussi une liste pour creole 1)
        elif self.version == 'creole1':
            try:
                value = self.dico.get_var(varname)[1]
            except:
                value = ''
        return value
 
    def set_value(self, value, invisible=False, force=False):
        if self.version == 'creole1':
            if type(value) == list:
                value = value[0]
            self.dico.set_value(value, invisible, force=force)
        elif self.version == 'creole2':
            try:
                self.dico.set_value(self.vars[self.index], value, force=force)
            except Exception, e:
                raise dico.InvalidDataError, e.args[0]

    def save(self, file='/dev/null', force=False, encode=False):
        if self.version == 'creole1':
            data = self.dico.save(file, encode=encode)
        elif self.version == 'creole2':
            # sauvegarde du fichier de valeurs
            self.dico.save_values(file,force)
            # on retourne le dictionnaire des valeurs pour recréer le ConfigParser
            data = [str(self.dico.store._sections)]
        return data

    def get_dict(self):
        # on envoie le dictionnaire sous forme de fichier texte
        data = []
        if self.version == 'creole2':
            for dic in self.dicos:
                f = open(dic)
                content = f.read()
                f.close()
                data.append(base64.encodestring(content))
            data.extend(self.save(force=True))
            return data
        elif self.version == 'creole1':
            return [base64.encodestring(self.save())]

    def get_menu(self, expert=False):
        """retourne les familles et les variables dans l'ordre de saisie 
        @param expert: mode de saisie (non expert par défaut)
        @return: renvoie une liste ordonnée de familles [famille, liste_vars]
        """
        menu = []
        if self.version == 'creole1':
            # on met toutes les variables (dans l'ordre de saisie) dans une famille "general".
            middle = len(self.dico.dictionnaire)/2
            end = len(self.dico.dictionnaire)-middle
            vars1 = [self.dico.dictionnaire[numvar][0] for numvar in range(middle)]
            vars2 = [self.dico.dictionnaire[middle + numvar][0] for numvar in range(end)]
            menu.append(['general', False, vars1])
            menu.append(['general_1', False, vars2])
#            vars =[self.dico.dictionnaire[numvar][0] for numvar in range(len(self.dico.dictionnaire))]
#            menu.append(['general', False, vars])
            
        else:
            # liste des familles normales
            for family in self.dico.family_order:
                data = self.dico.families[family]
                if not data['mode'] == 'expert':
                    # on récupère la liste des variables à afficher dans l'ordre
                    vars = data['vars']
                    menu.append([family.encode(charset), data['hidden'], vars])
            # liste des familles expert
            if expert == True:
                for family in self.dico.family_order:
                    data = self.dico.families[family]
                    if data['mode'] == 'expert':
                        vars = data['vars']
                        menu.append([family.encode(charset), data['hidden'], vars])
        return menu

    def parsedico(self):
        # renvoie la configuration sous forme d'un dictionnaire {variable:valeur}
        data = {}
        if self.version == 'creole1':
            for varname, values in self.dico.ParseDico().items():
                data[varname] = values[0]
        else:
            for varname in self.dico.variables.keys():
                data[varname] = ", ".join(self.dico.get_value(varname))
        return data


