#!/usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# init_base.py
#  
# Script d'insertion des données par défaut dans la base de données
# (types établissement et modules par défaut)
#       
###########################################################################

# insertion des types d'établissement
from zephir.backend import nomenclature
import xmlrpclib, getpass, os, sys, glob
from zephir.config import *
from creole import parsedico

# récupération du login admin
dictionnaire = parsedico.parse_dico()
admin_zephir = dictionnaire['admin_zephir']

def init_data(z):
    """mise en place des données de base (modules, types etablissements)"""

    # import des types d'établissement
    print "création des types d'établissement par défaut ...\n"

    for libelle in nomenclature.nomenclature_ramses.values():
        z.etabs.add_type(libelle)

    # création des modules
    print "création des modules Eole ...\n"

    res = z.modules.get_module()
    if res[0] == 0:
        modules_actuel = []
    else:
        modules_actuel = res[1]
        
    libelles = []

    for module in modules_actuel:
        libelles.append(module['libelle'])
        
    for module in liste_modules:
        if module in libelles:
            print("\nle module %s existe déjà !" % module)
        else:
            if module in OLD_MODULES:
                version = 1
            else:
                version = 2
            res,id_module=z.modules.add_module(module, version)
            if res == 0:
                print("\nerreur de création du module %s : %s\n" % (module,id_module))
            else:
                if version == 1:
                    # installation du dictionnaire
                    fichier = open("../dictionnaires/dico-%s" % module,"r")
                    buffer_dico = fichier.read()
                    fichier.close()
                    res, info = z.uucp.install_module(int(id_module),xmlrpclib.base64.encodestring(buffer_dico))
                else:
                    # Eole 2
                    dicos = {}
                    for dico in glob.glob("../dictionnaires/%s/*.xml" % module):
                        # lecture des dictionnaires et de leur nom
                        data = open(dico).read()
                        dicos[os.path.basename(dico)] = xmlrpclib.base64.encodestring(data)
                    res, info = z.uucp.install_module(int(id_module),dicos)
                if res == 0:
                    print("erreur d'envoi du dictionnaire pour le module %s" % module)

    print "\ninsertion des données de base terminée\n"

def check_admin():
    """vérifie que l'admin a bien des droits dans la base"""
    import psycopg2
    from zephir.backend.config import DB_PASSWD, DB_HOST, DB_USER, DB_NAME
    cx = psycopg2.connect(host=DB_HOST, database=DB_NAME, user=DB_USER, password=DB_PASSWD)
    db_cursor = cx.cursor()
    db_cursor.execute("""select droits from users where login='%s'""" % admin_zephir)
    res = db_cursor.fetchone()
    db_cursor.close()
    if res != None:
        admin_perms = eval(res[0])
    else:
        admin_perms = []
    if 1 in admin_perms and 4 in admin_perms:
        # droits minimums ok
        cx.close()
        sys.exit(0)
    else:
        # on ajoute les droits
        try:
            db_cursor = cx.cursor()
            if res == None:
                # pas d'infos existantes
                db_cursor.execute("""insert into users (login,mail_actif,sms_actif,droits) values ('%s',0,0,'[1,4]')""" % admin_zephir)
            else:
                if 1 not in admin_perms:
                    admin_perms.append(1)
                if 4 not in admin_perms:
                    admin_perms.append(4)
                # update du champs droits
                db_cursor.execute("""update users set droits='%s' where login='%s'""" % (str(admin_perms),admin_zephir))
            db_cursor.close()
        except:
            # pas d'admin défini ou erreur de maj des droits
            print "\nErreur de mise en place des droits de l'administrateur zephir !\n"
            cx.rollback()
            sys.exit(1)
        cx.commit()
        print "droits mis à jour pour %s\n" % admin_zephir

if __name__ == '__main__':
    if "--verify" in sys.argv:
        check_admin()
    else:
        # connexion à l'application zephir
        try:
            passwd = sys.argv[1]
        except:
            passwd = getpass.getpass("mot de passe de %s : " % admin_zephir)

        init_data(xmlrpclib.ServerProxy('https://%s:%s@%s:%s' % (admin_zephir, passwd, ADRESSE_ZEPHIR, PORT_ZEPHIR)))
